@@ -6,9 +6,12 @@ use warnings; # XXX -- development only
 use CGI;
 use Coro;
 use Coro::Channel;
-
 use Continuity::RequestHolder;
 
+# Accessors
+sub server :lvalue { $_[0]->{server} }
+sub debug_level :lvalue { $_[0]->{debug_level} }         # Debug level (integer)
+
 =head1 NAME
 
 Continuity::Mapper - Map a request onto a session
@@ -138,7 +141,7 @@ sub get_session_id_from_hit {
   my ($self, $request) = @_;
   my $session_id = '';
   my $sid;
-  $self->Continuity::debug(2,"        URI: ", $request->uri, "\n");
+  $self->Continuity::debug(2,"        URI: ", $request->uri);
 
   # IP based sessions
   if($self->{ip_session}) {
@@ -158,27 +161,29 @@ sub get_session_id_from_hit {
   # Query sessions
   if($self->{query_session}) {
     $sid = $request->param($self->{query_session}) || '';
-    $self->Continuity::debug(2,"    Session: got query '$sid'\n");
+    $self->Continuity::debug(2,"    Session: got query '$sid'");
   }
 
   # Cookie sessions
   if($self->{cookie_session}) {
-     # use Data::Dumper 'Dumper'; STDERR->print("request->headers->header(Cookie): ", Dumper($request->headers->header('Cookie')));
     my $cookie = $request->get_cookie($self->{cookie_session});
     $sid = $cookie if $cookie;
-    $self->Continuity::debug(2,"    Session: got cookie '$sid'\n") if $sid;
+    $self->Continuity::debug(2,"    Session: got cookie '$sid'") if $sid;
   }
 
   if(($self->{query_session} or $self->{cookie_session}) and ! $sid) {
       $sid = $self->{assign_session_id}->($request);
-      $self->Continuity::debug(2,"    New SID: $sid\n");
-      $request->set_cookie( CGI->cookie( -name => $self->{cookie_session}, -value => $sid, -expires => $self->{cookie_life}, ) ) if $self->{cookie_session};
-      # XXX somehow record the sid in the request object in case of query_session
+      $self->Continuity::debug(2,"    New SID: $sid");
+      $request->set_cookie( CGI->cookie(
+        -name    => $self->{cookie_session},
+        -value   => $sid,
+        -expires => $self->{cookie_life},
+      )) if $self->{cookie_session};
   }
 
   $session_id .= $sid if $sid;
 
-  $self->Continuity::debug(2," Session ID: ", $session_id, "\n");
+  $self->Continuity::debug(2," Session ID: ", $session_id);
 
   return $session_id;
 
@@ -204,21 +209,31 @@ sub map {
 
   $self->{sessions_last_access}->{$session_id} = time;
 
-  $self->Continuity::debug(2,"    Session: count " . (scalar keys %{$self->{sessions}}) . "\n");
+  $self->Continuity::debug(2,"    Session: count " . (scalar keys %{$self->{sessions}}));
 
   if( ! $self->{sessions}->{$session_id} ) {
-      $self->Continuity::debug(2,"    Session: No request queue for this session ($session_id), making a new one.\n");
+      $self->Continuity::debug(2,"    Session: No request queue for this session ($session_id), making a new one.");
       $self->{sessions}->{$session_id} = $self->new_request_queue($session_id);
   }
 
   my $request_queue = $self->{sessions}->{$session_id};
 
-  $self->exec_cont($request, $request_queue);
+  $self->enqueue($request, $request_queue);
 
   return $request;
 
 }
 
+=head2 $mapper->reap($age)
+
+Reap all sessions older than $age.
+
+Reaping is done through the 'immediate' execution request mechanism. A special
+request is sent to the session that the session executes instead of user code.
+The special request then called Coro::terminate to kill itself.
+
+=cut
+
 sub reap {
     my $self = shift;
     my $age = shift or die "pass reap a number of seconds";
@@ -226,22 +241,19 @@ sub reap {
     my $sessions_last_access = $self->{sessions_last_access};
     for my $session_id (keys %$sessions ) {
         next if $sessions_last_access->{$session_id} + $age > time;
-        warn "$session_id dies";
+        $self->Continuity::debug(2, "Session $session_id is being reaped!");
         my $request = do {
             package Continuity::Request::Death;
             use base 'Continuity::Request';
             sub immediate { Coro::terminate(0); }
             bless { }, __PACKAGE__;
         };
-        $self->exec_cont($request, $sessions->{$session_id});
+        $self->enqueue($request, $sessions->{$session_id});
         delete $sessions->{$session_id};
         delete $sessions_last_access->{$session_id};
     }
 }
 
-sub server :lvalue { $_[0]->{server} }
-sub debug_level :lvalue { $_[0]->{debug_level} }         # Debug level (integer)
-
 =head2 $request_queue = $mapper->new_request_queue($session_id)
 
 Returns a brand new session request queue, and starts a session to pull
@@ -253,11 +265,12 @@ sub new_request_queue {
   my $self = shift;
   my $session_id = shift or die;
 
-  # Create a request_queue, and hook the adaptor up to feed it
+  # Create a request_queue, and hook the adapter up to feed it
   my $request_queue = Coro::Channel->new();
   my $request_holder = Continuity::RequestHolder->new(
     request_queue => $request_queue,
     session_id    => $session_id,
+    debug_level   => $self->debug_level,
   );
 
   # async just puts the contents into the global event queue to be executed
@@ -268,7 +281,7 @@ sub new_request_queue {
 
     # If the callback exits, the session is over
     delete $self->{sessions}->{$session_id};
-    $self->Continuity::debug(1,"XXX debug: session $session_id closed\n");
+    $self->Continuity::debug(2,"Session $session_id closed");
   };
 
   return $request_queue;
@@ -284,7 +297,7 @@ implementation will optionally print the HTTP headers for you.
 
 =cut
 
-sub exec_cont {
+sub enqueue {
   my ($self, $request, $request_queue) = @_;
 
   # TODO: This might be one spot to hook STDOUT onto this request
@@ -293,7 +306,6 @@ sub exec_cont {
   $request_queue->put($request);
 
   # XXX needed for FastCGI (because it is blocking...)
-  # print STDERR "yielding to other things (for FCGI's sake)\n";
   cede;
 
   # select $prev_select;
